@@ -17,4 +17,10 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
+# pylint: disable=invalid-name
+
+"""Custom script for smuxi-frontend-gnome."""
+
+# https://gitlab.gnome.org/GNOME/orca/-/issues/358
+# ruff: noqa: F401
 from .script import Script
